import fs from 'fs';
import { copySync } from 'node-fs-extra';
import path from 'path';

/**
 * Get all subdirectories in a passed-in directory.
 * @param  {string} srcpath The path of the directory to search.
 * @return {array}         The subdirectories.
 */
function getDirectories(srcpath) {
  return fs
    .readdirSync(srcpath)
    .filter(file => fs.statSync(path.join(srcpath, file)).isDirectory());
}

/**
 * If an index.php file exists, copies it to current project, changing its name
 * to its node_modules directory name.
 * @param  {string} PHPfilename The file path.
 */
function maybeCopyIndexPHP(PHPfilename, directory) {
  if (fs.existsSync(PHPfilename)) {
    fs
      .createReadStream(PHPfilename)
      .pipe(
        fs.createWriteStream(
          `${process.env.PWD}/wp-autoload/from-node/${directory}.php`,
        ),
      );
  }
}

/**
 * If the node_modules directory has subdirectories, starting with `wp-`, copy
 * them into the current project.
 * @param  {string} directoryPath Path to the module directory.
 */
function maybeCopyNecessarySubdirectories(directoryPath) {
  const subdirectories = getDirectories(directoryPath);

  subdirectories.forEach((subdirectory) => {
    if (subdirectory.indexOf('wp-') === 0) {
      copySync(
        `${directoryPath}${subdirectory}`,
        `${process.env.PWD}/wp-autoload/from-node/${subdirectory}`,
      );
    }
  });
}

/**
 * Checks the contents of a directory and possibly copy files/folders into the
 * root directory.
 * @param  {string} directory The directory to check.
 */
function maybeCopyFromDirectory(directory) {
  const directoryPath = `${process.env.PWD}/node_modules/${directory}/`;
  const PHPfilename = path.normalize(`${directoryPath}index.php`);

  maybeCopyIndexPHP(PHPfilename, directory);
  maybeCopyNecessarySubdirectories(directoryPath);
}

/**
 * Runs the script.
 */
export default function copyIndexPHPFilesFromNodeModules() {
  const directories = getDirectories(
    path.normalize(`${process.env.PWD}/node_modules`),
  );

  if (!directories) {
    return;
  }

  directories.forEach(maybeCopyFromDirectory);
}
